package MyBlog::Controller::MediaCollection;

use strict;
use warnings;
use base qw(Catalyst::Controller::Atompub::Collection);

sub get_feed :Atompub(list) {
    my($self, $c) = @_;

    # フィード (XML::Atom::Feed) のひな型
    my $feed = $self->collection_resource->body;

    # medias テーブルからメンバリソースを取得する (更新日時の新しい順)
    my $rs = $c->model('DBIC::Medias')
               ->search({}, { order_by => 'edited DESC' });

    # フィードのひな形にメディアリンクエントリを追加する
    while (my $resource = $rs->next) {
        my $entry = XML::Atom::Entry->new(\$resource->entry_body);
        $feed->add_entry($entry);
    }

    # 成功したら true を返す
    return 1;
}

use MIME::Base64;
sub create_resource :Atompub(create) {
    my($self, $c) = @_;

    # スーパークラスが自動生成した URI と更新日時を取得する
    my $entry_uri = $self->media_link_entry->uri;
    my $media_uri = $self->media_resource->uri;
    my $edited    = $self->edited->epoch; # (UNIX time 形式に変換)

    # POST されたメディアリソースを取得する (バイナリデータ)
    my $media = $self->media_resource->body;

    # メディアリソースのメディアタイプ (image/jpeg など) を取得する
    my $media_type = $self->media_resource->type;

    # スーパークラスが自動生成したメディアリンクエントリ (XML::Atom::Entry) を取得する
    my $entry = $self->media_link_entry->body;

    # medias テーブルにメディアリソースとメディアリンクエントリを格納する
    $c->model('DBIC::Medias')->update_or_create({
        edited     => $edited,
        entry_uri  => $entry_uri,
        entry_body => $entry->as_xml,
        media_uri  => $media_uri,
        media_body => MIME::Base64::encode($media), # ASCIIに変換する
        media_type => $media_type,
    });

    # 成功したら true を返す
    return 1;
}

sub get_resource :Atompub(read) {
    my($self, $c) = @_;

    # リクエストされた URI
    my $uri = $c->req->uri;

    # medias テーブルからリソースを検索する
    my $rs = $c->model('DBIC::Medias')->search({
        '-or' => [
            { entry_uri => $uri },
            { media_uri => $uri },
        ]
    })->first;

    # メディアリンクエントリの URI であった場合
    if ($rs->entry_uri eq $uri) {
        # メディアリンクエントリをセットする
        $self->media_link_entry->body( XML::Atom::Entry->new(\$rs->entry_body) );
    }
    # メディアリソースの URI であった場合
    else {
        # メディアリソースをセットする
        $self->media_resource->body( MIME::Base64::decode($rs->media_body) );
        $self->media_resource->type( $rs->media_type );
    }

    # 成功したら true を返す
    return 1;
}

sub update_resource :Atompub(update) {
    my($self, $c) = @_;

    # リクエストされた URI
    my $uri = $c->req->uri;

    # medias テーブルからリソースを検索する
    my $rs = $c->model('DBIC::Medias')->search({
        '-or' => [
            { entry_uri => $uri },
            { media_uri => $uri },
        ]
    });

    # $vals には，更新するカラムの値をセットする
    my $vals = { edited => $self->edited->epoch };

    # メディアリンクエントリの URI であった場合
    if ($rs->entry_uri eq $uri) {
        # メディアリンクエントリをセットする
        $vals->{entry_body} = $self->media_link_entry->body->as_xml;
    }
    # メディアリソースの URI であった場合
    else {
        # メディアリンクエントリの更新日時 (app:edited) を更新し，セットする
        my $entry = XML::Atom::Entry->new(\$rs->entry_body);
           $entry->edited( $self->edited->w3c );
        $vals->{entry_body} = $entry->as_xml;

        # メディアリソースをセットする
        $vals->{media_body} = MIME::Base64::encode( $self->media_resource->body );
        $vals->{media_type} = $self->media_resource->type;
    }

    # 更新を実行する
    $rs->update($vals);

    # 成功したら true を返す
    return 1;
}

sub delete_resource :Atompub(delete) {
    my($self, $c) = @_;

    # リクエストされた URI
    my $uri = $c->req->uri;

    # medias テーブルからエントリを削除する
    my $rs = $c->model('DBIC::Medias')->search({
        '-or' => [
            { entry_uri => $uri },
            { media_uri => $uri },
        ]
    })->delete;

    # 成功したら true を返す
    return 1;
}

# Make EditURI by yourself (it will be automatically generated by default)
#sub make_edit_uri {
#    my($self, $c, @args) = @_;
#}

# Find version information of the Resource for caching and versioning
#sub find_version {
#    my($self, $c, $uri) = @_;
#}

=head1 NAME

MyBlog::Controller::MediaCollection - Atom Collection Resources

=head1 SYNOPSIS

See L<MyBlog>.

=head1 DESCRIPTION

Atom Collection Resources class.

=head1 AUTHOR

Takeru INOUE

=head1 LICENSE

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
